package org.wordpress.android.viewmodel.wpwebview

import androidx.annotation.DrawableRes
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.LifecycleRegistry
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModel
import org.wordpress.android.R
import org.wordpress.android.ui.WPWebViewUsageCategory
import org.wordpress.android.ui.utils.UiString.UiStringRes
import org.wordpress.android.util.NetworkUtilsWrapper
import org.wordpress.android.viewmodel.SingleLiveEvent
import org.wordpress.android.viewmodel.helpers.ConnectionStatus
import org.wordpress.android.viewmodel.helpers.ConnectionStatus.AVAILABLE
import org.wordpress.android.viewmodel.wpwebview.WPWebViewViewModel.PreviewMode.DEFAULT
import org.wordpress.android.viewmodel.wpwebview.WPWebViewViewModel.PreviewMode.DESKTOP
import org.wordpress.android.viewmodel.wpwebview.WPWebViewViewModel.WebPreviewUiState.WebPreviewContentUiState
import org.wordpress.android.viewmodel.wpwebview.WPWebViewViewModel.WebPreviewUiState.WebPreviewFullscreenProgressUiState
import org.wordpress.android.viewmodel.wpwebview.WPWebViewViewModel.WebPreviewUiState.WebPreviewFullscreenUiState.WebPreviewFullscreenErrorUiState
import javax.inject.Inject

class WPWebViewViewModel
@Inject constructor(
    private val networkUtils: NetworkUtilsWrapper,
    connectionStatus: LiveData<ConnectionStatus>
) : ViewModel(), LifecycleOwner {
    private var isStarted = false
    private var wpWebViewUsageCategory: WPWebViewUsageCategory = WPWebViewUsageCategory.WEBVIEW_STANDARD

    private val _uiState: MutableLiveData<WebPreviewUiState> = MutableLiveData()
    val uiState: LiveData<WebPreviewUiState> = _uiState
    private val _loadNeeded = SingleLiveEvent<Boolean>()
    val loadNeeded: LiveData<Boolean> = _loadNeeded

    private val _navigateBack = SingleLiveEvent<Unit>()
    val navigateBack: LiveData<Unit> = _navigateBack

    private val _navigateForward = SingleLiveEvent<Unit>()
    val navigateForward: LiveData<Unit> = _navigateForward

    private val _share = SingleLiveEvent<Unit>()
    val share: LiveData<Unit> = _share

    private val _openInExternalBrowser = SingleLiveEvent<Unit>()
    val openExternalBrowser: LiveData<Unit> = _openInExternalBrowser

    private val _previewModeSelector = MutableLiveData<PreviewModeSelectorStatus>()
    val previewModeSelector: LiveData<PreviewModeSelectorStatus> = _previewModeSelector

    private val _navbarUiState: MutableLiveData<NavBarUiState> = MutableLiveData()
    val navbarUiState: LiveData<NavBarUiState> = _navbarUiState

    private val _previewMode: MutableLiveData<PreviewMode> = MutableLiveData()
    val previewMode: LiveData<PreviewMode> = _previewMode

    private val lifecycleRegistry = LifecycleRegistry(this)
    override fun getLifecycle(): Lifecycle = lifecycleRegistry

    init {
        lifecycleRegistry.markState(Lifecycle.State.CREATED)
        connectionStatus.observe(this, Observer {
            if (it == AVAILABLE) {
                loadIfNecessary()
            }
        })
    }

    fun start(webViewUsageCategory: WPWebViewUsageCategory) {
        if (isStarted) {
            return
        }
        isStarted = true
        wpWebViewUsageCategory = webViewUsageCategory
        _navbarUiState.value = NavBarUiState(
                forwardNavigationEnabled = false,
                backNavigationEnabled = false,
                desktopPreviewHintVisible = false
        )
        _previewMode.value = DEFAULT
        _previewModeSelector.value = PreviewModeSelectorStatus(
                isVisible = false,
                isEnabled = false,
                selectedPreviewMode = DEFAULT
        )

        if (WPWebViewUsageCategory.isActionableDirectUsage(wpWebViewUsageCategory)) {
            updateUiState(WPWebViewUsageCategory.actionableDirectUsageToWebPreviewUiState(wpWebViewUsageCategory))
        } else if (networkUtils.isNetworkAvailable()) {
            updateUiState(WebPreviewFullscreenProgressUiState)
        } else {
            updateUiState(WebPreviewFullscreenErrorUiState())
        }
        lifecycleRegistry.markState(Lifecycle.State.STARTED)
    }

    override fun onCleared() {
        lifecycleRegistry.markState(Lifecycle.State.DESTROYED)
        super.onCleared()
    }

    private fun updateUiState(uiState: WebPreviewUiState) {
        _uiState.value = uiState
    }

    /**
     * Update the ui state if the Loading or Error screen is being shown.
     * In other words don't update it after a configuration change.
     */
    fun onUrlLoaded() {
        if (uiState.value !is WebPreviewContentUiState) {
            updateUiState(WebPreviewContentUiState)
            _previewModeSelector.value = _previewModeSelector.value?.copy(isEnabled = true)
        }
        _loadNeeded.value = false
    }

    /**
     * Update the ui state if the Loading or Success screen is being shown.
     */
    fun onReceivedError() {
        if (uiState.value is WebPreviewContentUiState) {
            return
        }
        if (uiState.value !is WebPreviewFullscreenErrorUiState) {
            updateUiState(WebPreviewFullscreenErrorUiState())
        }
        _loadNeeded.value = false
    }

    fun loadIfNecessary() {
        if (isActionableDirectUsage()) return

        if (uiState.value !is WebPreviewFullscreenProgressUiState &&
                uiState.value !is WebPreviewContentUiState
        ) {
            updateUiState(WebPreviewFullscreenProgressUiState)
            _loadNeeded.value = true
        }
    }

    fun isActionableDirectUsage() = WPWebViewUsageCategory.isActionableDirectUsage(wpWebViewUsageCategory)

    fun getMenuUiState() = wpWebViewUsageCategory.menuUiState

    fun navigateBack() {
        _navigateBack.call()
    }

    fun navigateForward() {
        _navigateForward.call()
    }

    fun toggleBackNavigation(isEnabled: Boolean) {
        _navbarUiState.value = navbarUiState.value!!.copy(backNavigationEnabled = isEnabled)
    }

    fun toggleForwardNavigation(isEnabled: Boolean) {
        _navbarUiState.value = navbarUiState.value!!.copy(forwardNavigationEnabled = isEnabled)
    }

    fun share() {
        _share.call()
    }

    fun openPageInExternalBrowser() {
        _openInExternalBrowser.call()
    }

    fun togglePreviewModeSelectorVisibility(isVisible: Boolean) {
        _previewModeSelector.value = PreviewModeSelectorStatus(isVisible, true, previewMode.value!!)
    }

    fun selectPreviewMode(selectedPreviewMode: PreviewMode) {
        if (previewMode.value != selectedPreviewMode) {
            _previewMode.value = selectedPreviewMode
            _navbarUiState.value =
                    navbarUiState.value!!.copy(desktopPreviewHintVisible = selectedPreviewMode == DESKTOP)
            updateUiState(WebPreviewFullscreenProgressUiState)
        }
    }

    data class NavBarUiState(
        val forwardNavigationEnabled: Boolean,
        val backNavigationEnabled: Boolean,
        val desktopPreviewHintVisible: Boolean
    )

    enum class PreviewMode {
        DEFAULT,
        DESKTOP
    }

    data class PreviewModeSelectorStatus(
        val isVisible: Boolean,
        val isEnabled: Boolean,
        val selectedPreviewMode: PreviewMode
    )

    sealed class WebPreviewUiState(
        val fullscreenProgressLayoutVisibility: Boolean = false,
        val actionableEmptyView: Boolean = false
    ) {
        object WebPreviewContentUiState : WebPreviewUiState()

        object WebPreviewFullscreenProgressUiState : WebPreviewUiState(
                fullscreenProgressLayoutVisibility = true
        )

        sealed class WebPreviewFullscreenUiState : WebPreviewUiState(actionableEmptyView = true) {
            abstract val imageRes: Int
            abstract val titleText: UiStringRes?
            abstract val subtitleText: UiStringRes?
            abstract val buttonVisibility: Boolean

            data class WebPreviewFullscreenErrorUiState(
                @DrawableRes
                override val imageRes: Int = R.drawable.img_illustration_cloud_off_152dp,
                override val titleText: UiStringRes = UiStringRes(R.string.error_browser_no_network),
                override val subtitleText: UiStringRes = UiStringRes(R.string.error_network_connection),
                override val buttonVisibility: Boolean = true
            ) : WebPreviewFullscreenUiState()

            object WebPreviewFullscreenNotAvailableUiState : WebPreviewFullscreenUiState() {
                @DrawableRes
                override val imageRes: Int = R.drawable.img_illustration_empty_results_216dp
                override val titleText: UiStringRes = UiStringRes(R.string.preview_unavailable_self_hosted_sites)
                override val subtitleText: UiStringRes? = null
                override val buttonVisibility: Boolean = false
            }
        }
    }
}
